==> mule-extension-ast-loader-1.8.11-20251017.011314-12-sources.jar: org/mule/runtime/ast/extension/internal/SourceElementAST.java <==
/*
 * Copyright 2023 Salesforce, Inc. All rights reserved.
 * The software in this package is published under the terms of the CPAL v1.0
 * license, a copy of which has been included with this distribution in the
 * LICENSE.txt file.
 */
package org.mule.runtime.ast.extension.internal;

import static org.mule.runtime.ast.extension.internal.IntrospectionTypeUtils.isSubType;

import static java.util.stream.Collectors.toList;

import org.mule.runtime.api.util.LazyValue;
import org.mule.runtime.extension.api.annotation.execution.OnError;
import org.mule.runtime.extension.api.annotation.execution.OnSuccess;
import org.mule.runtime.extension.api.annotation.execution.OnTerminate;
import org.mule.runtime.extension.api.annotation.source.OnBackPressure;
import org.mule.runtime.extension.api.runtime.source.Source;
import org.mule.runtime.module.extension.api.loader.java.type.MethodElement;
import org.mule.runtime.module.extension.api.loader.java.type.SourceElement;
import org.mule.runtime.module.extension.api.loader.java.type.Type;

import java.lang.annotation.Annotation;
import java.util.Arrays;
import java.util.List;
import java.util.Optional;

import javax.annotation.processing.ProcessingEnvironment;
import javax.lang.model.element.TypeElement;

/**
 * {@link SourceElement} implementation which works with the Java AST.
 *
 * @since 1.0
 */
public class SourceElementAST extends ASTType implements SourceElement {

  private LazyValue<List<Type>> sourceGenerics;

  public SourceElementAST(TypeElement typeElement, ProcessingEnvironment processingEnvironment) {
    super(typeElement, processingEnvironment);
    this.sourceGenerics =
        new LazyValue<>(() -> IntrospectionTypeUtils.getSuperTypeGenerics(this.typeElement,
                                                                          isLegacySource(processingEnvironment, typeElement)
                                                                              ? Source.class
                                                                              : org.mule.sdk.api.runtime.source.Source.class,
                                                                          processingEnvironment)
            .stream()
            .map(type -> new ASTType(type, processingEnvironment))
            .collect(toList()));
  }

  private boolean isLegacySource(ProcessingEnvironment processingEnvironment, TypeElement typeElement) {
    // TODO: MULE-19142: build test case
    return !isSubType(typeElement, org.mule.sdk.api.runtime.source.Source.class, processingEnvironment);
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public List<Type> getSuperClassGenerics() {
    return sourceGenerics.get();
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public Optional<MethodElement> getOnResponseMethod() {
    return findMethodAnnotatedWith(OnSuccess.class, org.mule.sdk.api.annotation.execution.OnSuccess.class);
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public Optional<MethodElement> getOnErrorMethod() {
    return findMethodAnnotatedWith(OnError.class, org.mule.sdk.api.annotation.execution.OnError.class);
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public Optional<MethodElement> getOnTerminateMethod() {
    return findMethodAnnotatedWith(OnTerminate.class, org.mule.sdk.api.annotation.execution.OnTerminate.class);
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public Optional<MethodElement> getOnBackPressureMethod() {
    return findMethodAnnotatedWith(OnBackPressure.class, org.mule.sdk.api.annotation.source.OnBackPressure.class);
  }

  private Optional<MethodElement> findMethodAnnotatedWith(Class<? extends Annotation>... annotationClasses) {
    return getMethods().stream()
        .filter(op -> Arrays.stream(annotationClasses).filter(annotationClass -> op.isAnnotatedWith(annotationClass)).findFirst()
            .isPresent())
        .map(elem -> (MethodElement) elem)
        .findFirst();
  }
}

==> mule-extension-ast-loader-1.10.0-20250923.144359-73-sources.jar: org/mule/runtime/ast/extension/internal/SourceElementAST.java <==
/*
 * Copyright 2023 Salesforce, Inc. All rights reserved.
 * The software in this package is published under the terms of the CPAL v1.0
 * license, a copy of which has been included with this distribution in the
 * LICENSE.txt file.
 */
package org.mule.runtime.ast.extension.internal;

import static org.mule.runtime.ast.extension.internal.IntrospectionTypeUtils.isSubType;

import static java.util.stream.Collectors.toList;

import org.mule.runtime.api.util.LazyValue;
import org.mule.runtime.extension.api.annotation.execution.OnError;
import org.mule.runtime.extension.api.annotation.execution.OnSuccess;
import org.mule.runtime.extension.api.annotation.execution.OnTerminate;
import org.mule.runtime.extension.api.annotation.source.OnBackPressure;
import org.mule.runtime.extension.api.runtime.source.Source;
import org.mule.runtime.module.extension.api.loader.java.type.MethodElement;
import org.mule.runtime.module.extension.api.loader.java.type.SourceElement;
import org.mule.runtime.module.extension.api.loader.java.type.Type;

import java.lang.annotation.Annotation;
import java.util.Arrays;
import java.util.List;
import java.util.Optional;

import javax.annotation.processing.ProcessingEnvironment;
import javax.lang.model.element.TypeElement;

/**
 * {@link SourceElement} implementation which works with the Java AST.
 *
 * @since 1.0
 */
public class SourceElementAST extends ASTType implements SourceElement {

  private final LazyValue<List<Type>> sourceGenerics;

  public SourceElementAST(TypeElement typeElement, ProcessingEnvironment processingEnvironment) {
    super(typeElement, processingEnvironment);
    this.sourceGenerics =
        new LazyValue<>(() -> IntrospectionTypeUtils.getSuperTypeGenerics(this.typeElement,
                                                                          isLegacySource(processingEnvironment, typeElement)
                                                                              ? Source.class
                                                                              : org.mule.sdk.api.runtime.source.Source.class,
                                                                          processingEnvironment)
            .stream()
            .map(type -> new ASTType(type, processingEnvironment))
            .collect(toList()));
  }

  private boolean isLegacySource(ProcessingEnvironment processingEnvironment, TypeElement typeElement) {
    // TODO: MULE-19142: build test case
    return !isSubType(typeElement, org.mule.sdk.api.runtime.source.Source.class, processingEnvironment);
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public List<Type> getSuperClassGenerics() {
    return sourceGenerics.get();
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public Optional<MethodElement> getOnResponseMethod() {
    return findMethodAnnotatedWith(OnSuccess.class, org.mule.sdk.api.annotation.execution.OnSuccess.class);
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public Optional<MethodElement> getOnErrorMethod() {
    return findMethodAnnotatedWith(OnError.class, org.mule.sdk.api.annotation.execution.OnError.class);
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public Optional<MethodElement> getOnTerminateMethod() {
    return findMethodAnnotatedWith(OnTerminate.class, org.mule.sdk.api.annotation.execution.OnTerminate.class);
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public Optional<MethodElement> getOnBackPressureMethod() {
    return findMethodAnnotatedWith(OnBackPressure.class, org.mule.sdk.api.annotation.source.OnBackPressure.class);
  }

  private Optional<MethodElement> findMethodAnnotatedWith(Class<? extends Annotation>... annotationClasses) {
    return getMethods().stream()
        .filter(op -> Arrays.stream(annotationClasses).anyMatch(op::isAnnotatedWith))
        .map(MethodElement.class::cast)
        .findFirst();
  }
}
